﻿
using System;
using System.IO;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data;
using System.Data.SQLite;
using Newtonsoft.Json;


namespace Backend
{
    public class Ops : IDisposable
    {

      private String dbFile = null;
      private String dbSource = null;
      private SQLiteConnection dbConn = null;
      private SQLiteCommand dbCmd = null;

      public Ops()
      {
         this.dbFile = ConfigurationManager.AppSettings["dbFileFullPath"];
         if (!File.Exists(this.dbFile))
            throw new FileNotFoundException("Missing db file: " + this.dbFile);
         this.dbSource = "DataSource={0}".FormatEx(this.dbFile);
         this.dbConn = new SQLiteConnection(this.dbSource);
      }

      /*
         client side methods
      */
      public String PostOrder(String ip, String jorder, String jadr)
      {
         try
         {
            Int64 rowid = 0L;
            Double cost = this.TotalCost(jorder);
            jorder = jorder.Replace("'", "''");
            jadr = jadr.Replace("'", "''");
            String sql = "insert into orders values('{0}', '{1}', '{2}', 1, {3}, datetime());".FormatEx(ip, jorder, jadr, cost);
            this.OpenConnection();
            this.dbCmd = new SQLiteCommand(sql, this.dbConn);
            if (this.dbCmd.ExecuteNonQuery() == 1)
               rowid = this.dbConn.LastInsertRowId;
            return rowid.ToString();

         }
         catch (SQLiteException exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);   
         }
         catch (Exception exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
      }

      public String MonitorOrder(Int64 rowid)
      {
         try
         {
            String sql = "select status from orders where rowid = {0};".FormatEx(rowid);
            this.OpenConnection();
            this.dbCmd = new SQLiteCommand(sql, this.dbConn);
            Object rval = this.dbCmd.ExecuteScalar();
            return rval.ToString();
         }
         catch (SQLiteException exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
         catch (Exception exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
      }

      // to be done by the students
      public String CancelOrder(Int64 rowid)
      {
         return "OK";
      }

      /*
         pizza shop methods 
      */
      public String UpdateOrder(Int64 rowid, Int32 status)
      {
         try
         {
            String sql = "update orders set status = {0} where rowid = {1};".FormatEx(status, rowid);
            this.OpenConnection();
            this.dbCmd = new SQLiteCommand(sql, this.dbConn);
            Int32 rows = this.dbCmd.ExecuteNonQuery();
            return rows.ToString();
         }
         catch (SQLiteException exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
         catch (Exception exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
      }

      public String ReadOpenOrdersCount()
      {
         try
         {
            String sql = "select count(*) from orders where status = 1;";
            this.OpenConnection();
            this.dbCmd = new SQLiteCommand(sql, this.dbConn);
            Object count = this.dbCmd.ExecuteScalar();
            return count.ToString();
         }
         catch (SQLiteException exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
         catch (Exception exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
      }

      public String ReadOpenOrders()
      {
         try
         {
            Int32 colOrder = 2; Int32 colAdr = 3;
            String sql = "select rowid,* from orders where status = 1 order by dtc asc;";
            List<Int64> rows = new List<Int64>();
            Int64 rowid = 0L; String order = null; String address = null;
            List<OpenOrder> orders = new List<OpenOrder>();

            this.OpenConnection();
            this.dbCmd = new SQLiteCommand(sql, this.dbConn);
            SQLiteDataReader reader = this.dbCmd.ExecuteReader();
            while (reader.Read())
            {
               rowid = reader.GetInt64(0);
               rows.Add(rowid);
               order = (reader.IsDBNull(colOrder)) ? "n/a" : reader.GetString(colOrder);
               address = (reader.IsDBNull(colAdr)) ? "n/a" : reader.GetString(colAdr);
               orders.Add(new OpenOrder(rowid, order, address));
            }
            reader.Close();

            // lock selected rows for processing
            String rowstolock = String.Join(",", rows);
            sql = "update orders set status = 2 where rowid in ({0});".FormatEx(rowstolock);
            using (this.dbCmd = new SQLiteCommand(sql, this.dbConn))
               this.dbCmd.ExecuteNonQuery();
            return JsonConvert.SerializeObject(orders);

         }
         catch (SQLiteException exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
         catch (Exception exp)
         {
            return this.GetErrorMsg(exp.Message, exp.StackTrace);
         }
      }

      public void Dispose()
      {
         if (this.dbCmd != null)
            this.dbCmd.Dispose();
         if (this.dbConn != null)
            this.dbConn.Dispose();
      }

      /*
         helper methods
      */
      public Double TotalCost(String json)
      {
         Double rval = 0d;
         List<Order> lst = JsonConvert.DeserializeObject<List<Order>>(json);
         foreach (Order o in lst)
            rval += Double.Parse(o.cost);
         return rval;
      }

      public void OpenConnection()
      {
         if (this.dbConn.State != ConnectionState.Open)
            this.dbConn.Open();
      }

      public String GetErrorMsg(String msg, String st)
      {
         return "ERROR: msg -> {0} ;;;; st -> {1}".FormatEx(msg, st);
      }

      public Int32 ClearDatabase()
      {
         String sql = "delete from orders;";
         this.OpenConnection();
         this.dbCmd = new SQLiteCommand(sql, this.dbConn);
         Int32 rows = this.dbCmd.ExecuteNonQuery();
         return rows;
      }

   }
}
