﻿
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Backend;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Backend.Tests
{
   [TestClass()]
   public class OpsTests
   {

      private String order = "[{\"lbl\":\"Pizza - Cheese - Regular\",\"cost\":\"18.99\"}," +
            "{\"lbl\":\"Pizza - Pepperoni - Large\",\"cost\":\"23.99\"}," +
            "{\"lbl\":\"Pizza - Meat Lover's - Regular\",\"cost\":\"19.99\"}]";
      private String address = "{\"street\":\"Morska 61\",\"city\":\"Gdynia\",\"zcode\":\"81-323\"}";
      private String ip0 = "127.0.0.100";
      private String ip1 = "188.252.6.2";

      [TestMethod()]
      public void PostOrderTest()
      {
         this.InitDatabase();
         Int64 rowid = 0L;
         using (Ops ops = new Ops())
         {
            String val = ops.PostOrder("100.100.100.100", this.order, this.address);
            rowid = Int64.Parse(val);
         }
         Assert.AreEqual(rowid, 15L);
      }

      [TestMethod()]
      public void MonitorOrderTest()
      {
         Int32 rval = 0;
         Random rand = new Random();
         Int32 status = rand.Next(0, 5);
         Int64 rowid = rand.Next(1, 14);
         this.InitDatabase();
         using(Ops ops = new Ops())
         {
            ops.UpdateOrder(rowid, status);
            rval = Int32.Parse(ops.MonitorOrder(rowid));
         }
         Assert.AreEqual(status, rval);
      }

      [TestMethod()]
      public void ReadOpenOrdersCountTest()
      {
         Int32 rval = 0;
         this.InitDatabase();
         using (Ops ops = new Ops())
            rval = Int32.Parse(ops.ReadOpenOrdersCount());
         Assert.AreEqual(rval, 14);
      }

      [TestMethod()]
      public void ReadOpenOrdersTest()
      {
         String jbuff = null;
         this.InitDatabase();
         using (Ops ops = new Ops())
            jbuff = ops.ReadOpenOrders();
         Assert.AreEqual(jbuff.Length, 3996);
      }

      [TestMethod()]
      public void TotalCostTest()
      {
         Double tc = 0d;
         using (Ops ops = new Ops())
            tc = ops.TotalCost(this.order);
         Assert.AreEqual(tc, 62.97d);
      }

      [TestMethod()]
      public void UpdateOrderTest()
      {
         Int32 rval = 0;
         Random rand = new Random();
         Int32 status = rand.Next(0, 5);
         Int64 rowid = rand.Next(1, 14);
         this.InitDatabase();
         using (Ops ops = new Ops())
            rval = Int32.Parse(ops.UpdateOrder(rowid, status));
         Assert.AreEqual(rval, 1);
      }

      public void InitDatabase()
      {
         using(Ops ops = new Ops())
         {
            ops.ClearDatabase();
            Int32 count = 0;
            while (count++ < 8)
               ops.PostOrder(this.ip0, this.order, this.address);
            count = 0;
            while(count++ < 6)
               ops.PostOrder(this.ip1, this.order, this.address);
         }
      }

   }
}